from typing import Dict, List, Iterator, Tuple
import copy
import os
import numpy as np
import cv2
from functools import reduce
from tqdm import tqdm

from blueness import module
from bluer_options.logger import log_list
from bluer_options import string
from bluer_objects import file
from bluer_objects import README
from bluer_objects.env import abcli_path_git

from bluer_ugv import NAME
from bluer_ugv.parts.part import Part
from bluer_ugv.README.consts import assets2
from bluer_ugv.logger import logger

NAME = module.name(__file__, NAME)


class PartDB:
    def __init__(self):
        self._db: Dict[str, Part] = {}

        self.url_prefix = f"{assets2}/bluer-ugv"

        self.path = os.path.join(
            abcli_path_git,
            "assets2/bluer-ugv",
        )

    def __iter__(self):
        return iter(self._db.values())

    def __setitem__(
        self,
        name: str,
        part: Part,
    ):
        assert isinstance(part, Part)

        self._db[name] = copy.deepcopy(part)
        self._db[name].name = name

    def __getitem__(self, name: str) -> Part:
        return self._db[name]

    def items(self) -> Iterator[Tuple[str, Part]]:
        return self._db.items()

    @property
    def README(self) -> List[str]:
        return sorted(
            [
                "- [{}](./{}.md).".format(
                    part.info[0],
                    part.name,
                )
                for part in self
            ]
        )

    def adjust(
        self,
        dryrun: bool = True,
        verbose: bool = False,
    ) -> bool:
        logger.info(
            "{}.adjust{}".format(
                NAME,
                " [dryrun]" if dryrun else "",
            )
        )

        list_of_filenames = reduce(
            lambda x, y: x + y,
            [
                [part.images[0]]
                for part_name, part in self._db.items()
                if part_name != "template" and part.images
            ],
            [],
        )
        log_list(logger, "adjusting", list_of_filenames, "images")

        max_width = 0
        max_height = 0
        for filename in tqdm(list_of_filenames):
            success, image = file.load_image(
                os.path.join(self.path, filename),
                log=verbose,
            )
            if not success:
                return success

            max_height = max(max_height, image.shape[0])
            max_width = max(max_width, image.shape[1])

        logger.info(f"size: {max_height} x {max_width}")

        for filename in tqdm(list_of_filenames):
            success, image = file.load_image(
                os.path.join(self.path, filename),
                log=verbose,
            )
            if not success:
                return success

            if image.shape[0] == max_height and image.shape[1] == max_width:
                logger.info("✅")
                continue

            image = image[:, :, :3]

            scale = min(
                max_height / image.shape[0],
                max_width / image.shape[1],
            )
            image = cv2.resize(
                image,
                dsize=(
                    int(scale * image.shape[1]),
                    int(scale * image.shape[0]),
                ),
                interpolation=cv2.INTER_LINEAR,
            )

            padded_image = (
                np.ones(
                    (max_height, max_width, 3),
                    dtype=np.uint8,
                )
                * 255
            )

            y_offset = (max_height - image.shape[0]) // 2
            x_offset = (max_width - image.shape[1]) // 2

            padded_image[
                y_offset : y_offset + image.shape[0],
                x_offset : x_offset + image.shape[1],
            ] = image

            if not dryrun:
                if not file.save_image(
                    os.path.join(self.path, filename),
                    padded_image,
                    log=verbose,
                ):
                    return False

        return True

    def as_images(
        self,
        dict_of_parts: Dict[str, str],
        reference: str = "../../parts",
    ) -> List[str]:
        return README.Items(
            [
                {
                    "name": self._db[part_name].info[0],
                    "marquee": self._db[part_name].image_url(
                        url_prefix=self.url_prefix
                    ),
                    "description": description,
                    "url": f"{reference}/{part_name}.md",
                }
                for part_name, description in dict_of_parts.items()
            ],
            sort=True,
        )

    def as_list(
        self,
        dict_of_parts: Dict[str, str],
        reference: str = "../../parts",
        log: bool = True,
    ) -> List[str]:
        if log:
            logger.info(
                "{}.as_list: {}".format(
                    self.__class__.__name__,
                    ", ".join(dict_of_parts.keys()),
                )
            )

        for part_name in dict_of_parts:
            if part_name not in self._db:
                logger.error(f"{part_name}: part not found.")
                assert False

        return sorted(
            [
                (
                    "1. [{}{}]({}).".format(
                        self._db[part_name].info[0],
                        ": {}".format(description) if description else "",
                        f"{reference}/{part_name}.md",
                    )
                )
                for part_name, description in dict_of_parts.items()
            ]
        )



from bluer_objects import README

from bluer_ugv.README.consts import assets2

items = README.Items(
    [
        {
            "name": "bluer_swallow",
            "marquee": f"{assets2}/bluer-swallow/20250701_2206342_1.gif?raw=true",
            "description": "based on power wheels.",
            "url": "./bluer_ugv/docs/bluer_swallow",
        },
        {
            "name": "bluer_sparrow",
            "marquee": f"{assets2}/bluer-sparrow/20250818_191815.jpg?raw=true",
            "description": "bluer_swallow's little sister.",
            "url": "./bluer_ugv/docs/bluer_sparrow",
        },
        {
            "name": "bluer_robin",
            "marquee": f"{assets2}/bluer-robin/20250723_095155~2_1.gif?raw=true",
            "description": "remote control car kit for teenagers.",
            "url": "./bluer_ugv/docs/bluer_robin",
        },
        {
            "name": "bluer_eagle",
            "marquee": f"{assets2}/bluer-eagle/file_0000000007986246b45343b0c06325dd.png?raw=true",
            "description": "a remotely controlled ballon.",
            "url": "./bluer_ugv/docs/bluer_eagle",
        },
        {
            "name": "bluer-fire",
            "marquee": "https://github.com/kamangir/assets/raw/main/bluer-ugv/bluer-fire.png?raw=true",
            "description": "based on a used car.",
            "url": "./bluer_ugv/docs/bluer_fire",
        },
        {
            "name": "bluer-beast",
            "marquee": "https://github.com/waveshareteam/ugv_rpi/raw/main/media/UGV-Rover-details-23.jpg",
            "description": "based on [UGV Beast PI ROS2](https://www.waveshare.com/wiki/UGV_Beast_PI_ROS2).",
            "url": "./bluer_ugv/docs/bluer_beast",
        },
    ]
)



dict_of_parts = {
    "DC-gearboxed-motor-12V-120RPM": "proposed, not used.",
    "dc-motor-12-VDC-45W": "type 2, 2 x right + 2 x left",
    "ultrasonic-sensor": "4 x",
    "resistor": "4 x 1 kΩ + 4 x 2 kΩ + 4 x 330-470 Ω",
    "LED": "4 x blue",
}



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets, assets2_bluer_swallow

items = ImageItems(
    {
        f"{assets}/bluer-ugv/bluer-light.png": "",
        "../../../diagrams/bluer_swallow/3d-design.png": "../../../diagrams/bluer_swallow/3d-design.stl",
        f"{assets2_bluer_swallow}/20250605_180136.jpg": "",
        f"{assets2_bluer_swallow}/20250608_144453.jpg": "",
        f"{assets2_bluer_swallow}/20250616_134654.jpg": "",
        f"{assets2_bluer_swallow}/20250616_145049.jpg": "",
        f"{assets2_bluer_swallow}/20250618_102816~2_1.gif": "",
        f"{assets2_bluer_swallow}/20250618_122604.jpg": "",
        "../../../diagrams/bluer_swallow/cover.png": "../../../diagrams/bluer_swallow/cover.stl",
        f"{assets2_bluer_swallow}/20250629_123616.jpg": "",
        f"{assets2_bluer_swallow}/20250630_214923.jpg": "",
        f"{assets2_bluer_swallow}/20250701_2206342_1.gif": "",
        f"{assets2_bluer_swallow}/20250707_122000.jpg": "",
        f"{assets2_bluer_swallow}/20250707_182818.jpg": "",
        f"{assets2_bluer_swallow}/20250709_111955.jpg": "",
    }
)



algo_docs = algo_docs = (
    "https://github.com/kamangir/bluer-algo/blob/main/bluer_algo/docs"
)

assets = "https://github.com/kamangir/assets/raw/main"

assets2 = "https://github.com/kamangir/assets2/raw/main"

assets2_bluer_eagle = f"{assets2}/bluer-eagle"
assets2_bluer_fire = f"{assets2}/bluer-fire"
assets2_bluer_robin = f"{assets2}/bluer-robin"
assets2_bluer_swallow = f"{assets2}/bluer-swallow"
assets2_bluer_sparrow = f"{assets2}/bluer-sparrow"



from blueness import module
import numpy as np

from bluer_options.terminal import hr
from bluer_algo.socket.classes import SocketComm
from bluer_algo.tracker.classes.target import Target

from bluer_ugv import NAME
from bluer_ugv.logger import logger


NAME = module.name(__file__, NAME)


def select_target(
    host: str,
    loop: bool = True,
) -> bool:
    logger.info(
        "{}.select_target on {}{}".format(
            NAME,
            host,
            " on a loop." if loop else "",
        )
    )

    try:
        while loop:
            socket = SocketComm.listen_on()
            success, image = socket.receive_data(np.ndarray)
            if not success:
                return success

            success, track_window = Target.select(
                image,
                title=f"select target on {host} ...",
            )
            if not success:
                return success

            socket = SocketComm.connect_to(host)
            if not socket.send_data(track_window):
                return False

            hr_line = hr(
                width=12,
                mono=True,
            )
            logger.info(f"{hr_line} Ctrl+C to exit {hr_line}")
    except KeyboardInterrupt:
        logger.info("Ctrl+C, stopping.")

    return True



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_ugv import NAME
from bluer_ugv.parts.db import db_of_parts
from bluer_ugv.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="adjust",
)
parser.add_argument(
    "--dryrun",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "adjust":
    success = db_of_parts.adjust(
        dryrun=args.dryrun == 1,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets2_bluer_robin

items = ImageItems(
    {
        f"{assets2_bluer_robin}/20250712_114819.jpg": "",
        "../../../diagrams/bluer_robin/wiring.png": "../../../diagrams/bluer_robin/wiring.svg",
        f"{assets2_bluer_robin}/20250713_172325.jpg": "",
        f"{assets2_bluer_robin}/20250713_172413.jpg": "",
        f"{assets2_bluer_robin}/20250713_172442_1.gif": "",
        f"{assets2_bluer_robin}/20250723_095022.jpg": "",
        f"{assets2_bluer_robin}/20250723_095155~2_1.gif": "",
        f"{assets2_bluer_robin}/20250728_112123.jpg": "",
        f"{assets2_bluer_robin}/20250726_134024.jpg": "",
        f"{assets2_bluer_robin}/20250807_103534.jpg": "",
        f"{assets2_bluer_robin}/20250807_171354.jpg": "",
        f"{assets2_bluer_robin}/20250813_172023.jpg": "",
    }
)



from typing import List

from bluer_options.terminal import show_usage


def help_select_target(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--host <hostname>]",
        "[--loop 0]",
    ]

    return show_usage(
        [
            "@swallow",
            "select_target",
        ]
        + args,
        "bluer-plugin leaf <object-name>.",
        mono=mono,
    )



from bluer_ugv.parts.part import Part
from bluer_ugv.parts.part_db import PartDB

db_of_parts: PartDB = PartDB()

db_of_parts["resistor"] = Part(
    info=[
        "Resistor, 1/4 watt, 5% tolerance",
    ],
    images=["resistor.png"],
)

db_of_parts["4-ch-transceiver"] = Part(
    info=[
        "4-channel transmitter and receiver",
        "source: [digikala](https://www.digikala.com/product/dkp-11037586/%DA%AF%DB%8C%D8%B1%D9%86%D8%AF%D9%87-%D9%88-%D9%81%D8%B1%D8%B3%D8%AA%D9%86%D8%AF%D9%87-%D9%85%D8%A7%D8%B4%DB%8C%D9%86-%DA%A9%D9%86%D8%AA%D8%B1%D9%84%DB%8C-%D9%85%D8%AF%D9%84-4ch-led/)",
        "voltages: receiver 6 VDC,  transmitter 3 VDC",
    ],
    images="4-channel-remote-control.png",
)

db_of_parts["470-mF"] = Part(
    info=[
        "capacitor, 470 μF to 1000 μF, 16 V or 25 V, Electrolytic, 105 °C rated if possible."
    ],
    images=["capacitor.png"],
)

db_of_parts["BTS7960"] = Part(
    info=[
        "43 A, H-Bridge Motor Driver",
        "specs: [BTS7960](https://www.handsontec.com/dataspecs/module/BTS7960%20Motor%20Driver.pdf)",
    ],
    images="bts7960.jpg",
)

db_of_parts["dc-motor-12-VDC-45W"] = Part(
    info=[
        "12 VDC motor, 20-45 W",
        "type 1: 9,000 RPM, output ~60 RPM",
        "type 2: 10,000 RPM, output 72 RPM",
        "https://parsbike.com/product/%D9%85%D9%88%D8%AA%D9%88%D8%B1-%DA%AF%DB%8C%D8%B1%D8%A8%DA%A9%D8%B3-%D9%85%D8%A7%D8%B4%DB%8C%D9%86-%D8%B4%D8%A7%D8%B1%DA%98%DB%8C-%D9%88-%D9%85%D9%88%D8%AA%D9%88%D8%B1-%D8%B4%D8%A7%D8%B1%DA%98%DB%8C/",
    ],
    images=[
        "gearbox1.jpg",
        "gearbox2.jpg",
        "gearbox3.jpg",
        "gearbox4.jpg",
        "gearbox5.jpg",
        "gearbox6.jpg",
        "gearbox7.jpg",
        "gearbox8.jpg",
    ],
)

db_of_parts["LED"] = Part(
    info=[
        "LED, ~2 V forward voltage, 10-20 mA",
    ],
    images=["led.png"],
)

db_of_parts["Polyfuse"] = Part(
    info=[
        "Polyfuse, 1.1 A hold, 2.2 A trip, 16 V, resettable, through-hole, e.g., MF-R110",
    ],
    images=["polyfuse.png"],
)

db_of_parts["rpi3bp"] = Part(
    info=[
        "Raspberry Pi 3B+",
    ],
    images=[
        "rpi3bplus.png",
        "gpio-pinout.png",
    ],
)

db_of_parts["SLA-Battery"] = Part(
    info=[
        "Rechargeable sealed lead acid battery, 12 V, 7 Ah",
    ],
    images=[
        "battery.png",
    ],
)

db_of_parts["TVS-diode"] = Part(
    info=[
        "TVS diode, unidirectional, 600 W, 6.8 V clamp, e.g. P6KE6.8A, DO-15 package",
    ],
    images=[
        "TVSdiode.png",
    ],
)

db_of_parts["XL4015"] = Part(
    info=[
        "12 VDC -> 5 VDC, 4A",
        "specs: [XL4015](https://www.handsontec.com/dataspecs/module/XL4015-5A-PS.pdf)",
    ],
    images=[
        "XL4015.png",
    ],
)

db_of_parts["rpi-camera"] = Part(
    info=[
        "Raspberry Pi Camera, V1.3"
        "https://www.raspberrypi.com/documentation/accessories/camera.html",
    ],
    images=[
        "rpi-camera.jpg",
    ],
)

db_of_parts["DC-gearboxed-motor-12V-120RPM"] = Part(
    info=[
        "Gearboxed DC Motor, 12 V (3-24 V), 3A, 120 RPM, 1:91, 15 Kg cm",
        "[GM6558](https://www.landaelectronic.com/product/%d9%85%d9%88%d8%aa%d9%88%d8%b1-dc-%da%af%db%8c%d8%b1%d8%a8%da%a9%d8%b3-%d8%ad%d9%84%d8%b2%d9%88%d9%86%db%8c-gm6558/)",
    ],
    images=[
        "GM6558/01.jpg",
        "GM6558/02.jpg",
        "GM6558/03.jpg",
        "GM6558/04.jpg",
        "GM6558/measurements.jpg",
        "GM6558/specs.png",
    ],
)

db_of_parts["2xAA-battery-holder"] = Part(
    info=[
        "2 x AA battery holder",
    ],
    images=[
        "2xAA-battery-holder.jpg",
    ],
)

db_of_parts["4xAA-battery-holder"] = Part(
    info=[
        "4 x AA battery holder",
    ],
    images=[
        "4xAA-battery-holder.jpg",
    ],
)

db_of_parts["PCB-double-9x7"] = Part(
    info=[
        "double-sided PCB, 9 cm x 7 cm",
    ],
    images=[
        "PCB-double-9x7.jpeg",
    ],
)

db_of_parts["PCB-single-14x9_5"] = Part(
    info=[
        "single-sided PCB, 14 cm x 9.5 cm",
    ],
    images=[
        "pcb-14x9_5cm.jpg",
    ],
)

db_of_parts["pushbutton"] = Part(
    info=[
        "push button",
    ],
    images=[
        "pushbutton.png",
    ],
)

db_of_parts["yellow-gearbox-dc-motor"] = Part(
    info=[
        "gearboxed DC motor, 6V DC",
    ],
    images=[
        "yellow-gearbox-dc-motor.png",
    ],
)

db_of_parts["yellow-wheels"] = Part(
    info=[
        "wheels for gearboxed DC motor",
    ],
    images=[
        "yellow-wheels.jpg",
    ],
)

db_of_parts["36v-hub-motor"] = Part(
    info=[
        "36V DC hub motor, 350 W, front, no gearbox",
        "[source](https://samamotor.ir/%D9%87%D8%A7%D8%A8-%D9%85%D9%88%D8%AA%D9%88%D8%B1/5105-%D9%87%D8%A7%D8%A8-%D9%85%D9%88%D8%AA%D9%88%D8%B1-350-%D9%88%D8%A7%D8%AA-36-%D9%88%D9%84%D8%AA-%D8%A8%D8%AF%D9%88%D9%86-%DA%AF%DB%8C%D8%B1%D8%A8%DA%A9%D8%B3-%D8%AF%D9%88%DA%86%D8%B1%D8%AE%D9%87-%D9%85%D8%AE%D8%B5%D9%88%D8%B5-%DA%86%D8%B1%D8%AE-%D8%AC%D9%84%D9%88-.html)",
    ],
    images=[
        "36v-hub-motor.jpg",
    ],
)

db_of_parts["brushless-350w-drive"] = Part(
    info=[
        "brushless drive, 36 - 48 V DC, 350 W, sine wave, silent",
        "[source](https://samamotor.ir/%D8%AF%D8%B1%D8%A7%DB%8C%D9%88%D8%B1-%D9%85%D9%88%D8%AA%D9%88%D8%B1-%D8%A8%D8%B1%D8%A7%D8%B4%D9%84%D8%B3-bldc/4821-%D8%AF%D8%B1%D8%A7%DB%8C%D9%88%D8%B1-%D8%A8%D8%B1%D8%A7%D8%B4%D9%84%D8%B3-36-48-%D9%88%D9%84%D8%AA-350-%D9%88%D8%A7%D8%AA-sine-wave-silent.html)",
    ],
    images=[
        "brushless-350w-drive.jpg",
    ],
)

db_of_parts["LJ-6V-battery"] = Part(
    info=[
        "6V DC (4 cell) NICD battery",
        "https://www.digikala.com/product/dkp-3213588/%C3%98/",
    ],
    images=[
        "LJ-6V-battery.jpg",
    ],
)


db_of_parts["USB-charger-NICD-6V"] = Part(
    info=[
        "6V DC charger for NICD batteries",
        "https://www.digikala.com/product/dkp-5977954/%D8%B4%D8%A7%D8%B1%DA%98%D8%B1-%D8%A8%D8%A7%D8%AA%D8%B1%DB%8C-%D9%85%D8%A7%D8%B4%DB%8C%D9%86-%DA%A9%D9%86%D8%AA%D8%B1%D9%84%DB%8C-%D9%85%D8%AF%D9%84-6-%D9%88%D9%84%D8%AA-%DA%A9%D8%AF-6v-usb-sm-%D8%A8%D9%87-%D9%87%D9%85%D8%B1%D8%A7%D9%87-%D8%B3%D9%88%DA%A9%D8%AA-sm-%D8%AF%D9%88-%D9%BE%DB%8C%D9%86/",
    ],
    images=[
        "USB-charger-NICD-6V-1.jpg",
        "USB-charger-NICD-6V-2.jpg",
    ],
)

db_of_parts["L-1x2"] = Part(
    info=[
        "L 1x2",
        "https://robotexiran.com/product/%d8%a8%d8%b3%d8%aa-21-l/",
    ],
    images=[
        "L-1x2-1.jpg",
        "L-1x2-2.jpg",
    ],
)

db_of_parts["shaft-10cm"] = Part(
    info=[
        "shaft, 10 cm",
        "https://robotexiran.com/product/%d9%85%d8%ad%d9%88%d8%b1-10cm/",
    ],
    images=[
        "shaft-10cm.jpg",
    ],
)

db_of_parts["M3"] = Part(
    info=[
        "M3 nuts and bolts",
    ],
    images=[
        "M3.jpg",
    ],
)

db_of_parts["front-connector"] = Part(
    info=[
        "front connector",
    ],
    images=[
        "front-connector.jpg",
    ],
)

db_of_parts["front-wheels"] = Part(
    info=[
        "front wheels",
    ],
    images=[
        "front-wheels.jpg",
    ],
)

db_of_parts["wheel"] = Part(
    info=[
        "power wheel wheels",
        "https://sarobatic.ir/product/%DA%86%D8%B1%D8%AE-%D8%A8%D8%B2%D8%B1%DA%AF-%D8%B9%D9%82%D8%A8-%D9%85%D8%A7%D8%B4%DB%8C%D9%86-%D8%B4%D8%A7%D8%B1%DA%98%DB%8C-%D8%A7%D8%B3%D8%AA%D9%88%DA%A9/",
        "https://toys-repair.ir/product/2768/",
    ],
    images=[
        "wheel1.jpg",
        "wheel4.jpg",
        "wheel3.jpg",
    ],
)

db_of_parts["ultrasonic-sensor"] = Part(
    info=[
        "HC-SR04",
        "[datasheet](https://cdn.sparkfun.com/datasheets/Sensors/Proximity/HCSR04.pdf)",
        "1m ~= 6ms",
        "fov = 15 - 30 deg",
    ],
    images=[
        "HC-SR04.jpg",
    ],
)

db_of_parts["template"] = Part(
    info=[
        "template",
    ],
    images=[
        "template.jpg",
    ],
)



from bluer_objects.README.items import ImageItems

from bluer_ugv.parts.db import db_of_parts
from bluer_ugv.swallow.README import items
from bluer_ugv.swallow.parts import dict_of_parts
from bluer_ugv.README.consts import algo_docs, assets, assets2_bluer_swallow

docs = [
    {
        "items": items,
        "path": "../docs/bluer_swallow",
    },
    {
        "path": "../docs/bluer_swallow/analog",
        "items": ImageItems(
            {
                "../../../../diagrams/bluer_swallow/analog.png": "../../../../diagrams/bluer_swallow/analog.svg",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital",
    },
    {
        "path": "../docs/bluer_swallow/digital/design",
    },
    {
        "path": "../docs/bluer_swallow/digital/design/operation.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/design/parts.md",
        "items": db_of_parts.as_images(
            dict_of_parts,
            reference="../../../parts",
        ),
        "macros": {
            "parts:::": db_of_parts.as_list(
                dict_of_parts,
                reference="../../../parts",
                log=False,
            ),
        },
    },
    {
        "path": "../docs/bluer_swallow/digital/design/terraform.md",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/20250611_100917.jpg": "",
                f"{assets2_bluer_swallow}/lab.png": "",
                f"{assets2_bluer_swallow}/lab2.png": "",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/design/steering-over-current-detection.md",
        "items": ImageItems(
            {
                "../../../../../diagrams/bluer_swallow/steering-over-current.png": "../../../../../diagrams/bluer_swallow/steering-over-current.svg",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/design/rpi-pinout.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset",
        "items": ImageItems(
            {
                f"{assets}/swallow-dataset-2025-07-11-10-53-04-n3oybs/grid.png": "./digital/dataset/combination/validation.md",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/collection",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/2025-07-08-13-09-38-so54ao.png": "",
                f"{assets2_bluer_swallow}/2025-07-09-11-20-27-4qf255-000-2.png": "",
                f"{assets2_bluer_swallow}/2025-07-09-11-18-07-azy27w.png": f"{algo_docs}/image_classifier/dataset/sequence.md",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/collection/validation.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/collection/one.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/combination"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/combination/validation.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/combination/one.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/dataset/review.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/model",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/model/validation.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/autonomous-driving/model/one.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/target-tracking",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/target-selection.png": f"{algo_docs}/socket.md",
            }
        ),
    },
]



from bluer_ugv.help.swallow.dataset import help_functions as help_dataset
from bluer_ugv.help.swallow.env import help_functions as help_env
from bluer_ugv.help.swallow.select_target import help_select_target

help_functions = {
    "dataset": help_dataset,
    "env": help_env,
    "select_target": help_select_target,
}



dict_of_parts = {
    "rpi3bp": "",
    "dc-motor-12-VDC-45W": "type 1, 2 x rear + steering",
    "SLA-Battery": "",
    "XL4015": "",
    "BTS7960": "rear drive (2 motors) & steering drive (1 motor)",
    "470-mF": "",
    "Polyfuse": "optional",
    "TVS-diode": "",
    "resistor": "3 x 330-470 Ω",
    "LED": "green + red + yellow",
    "rpi-camera": "",
    "PCB-single-14x9_5": "2 x",
    "PCB-double-9x7": "",
    "pushbutton": "",
    "wheel": "4 x ",
}



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets2_bluer_sparrow

items = ImageItems(
    {
        f"{assets2_bluer_sparrow}/20250722_174115-2.jpg": "",
        f"{assets2_bluer_sparrow}/20250818_191815.jpg": "",
    }
)



from bluer_ugv.README.consts import assets2

NAME = "bluer_ugv"

ICON = "🐬"

DESCRIPTION = f"{ICON} AI x ROS."

VERSION = "6.709.1"

REPO_NAME = "bluer-ugv"

MARQUEE = f"{assets2}/bluer-swallow/20250701_2206342_1.gif"

ALIAS = "@ugv"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from bluer_ugv.parts.db import db_of_parts
from bluer_ugv.sparrow.README import items
from bluer_ugv.sparrow.parts import dict_of_parts

docs = [
    {
        "items": items,
        "path": "../docs/bluer_sparrow",
        "cols": 2,
    },
    {"path": "../docs/bluer_sparrow/design"},
    {"path": "../docs/bluer_sparrow/design/specs.md"},
    {
        "path": "../docs/bluer_sparrow/design/parts.md",
        "items": db_of_parts.as_images(
            dict_of_parts,
            reference="../../parts",
        ),
        "macros": {
            "parts:::": db_of_parts.as_list(
                dict_of_parts,
                reference="../../parts",
                log=False,
            ),
        },
    },
    {
        "path": "../docs/bluer_sparrow/algo",
    },
    {
        "path": "../docs/bluer_sparrow/algo/target-detection",
    },
]



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets2_bluer_eagle

items = ImageItems(
    {
        f"{assets2_bluer_eagle}/file_0000000007986246b45343b0c06325dd.png": "",
        f"{assets2_bluer_eagle}/20250727_182113.jpg": "",
        f"{assets2_bluer_eagle}/20250726_171953.jpg": "",
    }
)



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_cp(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@swallow",
            "env",
            "cp",
            "[<env-name>]",
        ],
        "cp swallow swallow-raspbian-<env-name>.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@swallow",
            "env",
            "list",
        ],
        "list swallow envs.",
        mono=mono,
    )


help_functions = {
    "cp": help_cp,
    "list": help_list,
}



import keyboard

from bluer_sbc.session.functions import reply_to_bash

from bluer_ugv.swallow.session.classical.setpoint import ClassicalSetPoint
from bluer_ugv.swallow.session.classical.mode import OperationMode
from bluer_ugv import env
from bluer_ugv.logger import logger

bash_keys = {
    "i": "exit",
    "o": "shutdown",
    "p": "reboot",
    "u": "update",
}


class ClassicalKeyboard:
    def __init__(
        self,
        setpoint: ClassicalSetPoint,
    ):
        logger.info(
            "{}: {}".format(
                self.__class__.__name__,
                ", ".join(
                    [f"{key}:{action}" for key, action in bash_keys.items()],
                ),
            )
        )

        self.last_key: str = ""
        self.setpoint = setpoint

        self.mode = OperationMode.NONE

    def update(self) -> bool:
        self.last_key = ""

        mode = self.mode

        for key, event in bash_keys.items():
            if keyboard.is_pressed(key):
                reply_to_bash(event)
                return False

        if keyboard.is_pressed(" "):
            self.setpoint.stop()

        if keyboard.is_pressed("x"):
            self.setpoint.start()

        if keyboard.is_pressed("a"):
            self.last_key = "a"
            self.setpoint.put(
                what="steering",
                value=env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
            )
        elif keyboard.is_pressed("d"):
            self.last_key = "d"
            self.setpoint.put(
                what="steering",
                value=-env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
            )
        else:
            self.setpoint.put(
                what="steering",
                value=0,
                log=False,
            )

        if keyboard.is_pressed("s"):
            self.setpoint.put(
                what="speed",
                value=self.setpoint.get(what="speed") - 10,
            )

        if keyboard.is_pressed("w"):
            self.setpoint.put(
                what="speed",
                value=self.setpoint.get(what="speed") + 10,
            )

        if keyboard.is_pressed("y"):
            self.mode = OperationMode.NONE

        if keyboard.is_pressed("t"):
            self.mode = OperationMode.TRAINING

        if keyboard.is_pressed("g"):
            self.mode = OperationMode.ACTION

        if mode != self.mode:
            logger.info("mode: {}.".format(self.mode.name.lower()))

        return True



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets2_bluer_swallow


items = ImageItems(
    {
        "https://github.com/kamangir/bluer-ugv/raw/main/diagrams/bluer_swallow/digital.png": "https://github.com/kamangir/bluer-ugv/blob/main/diagrams/bluer_swallow/digital.svg",
        f"{assets2_bluer_swallow}/20250609_164433.jpg": "",
        f"{assets2_bluer_swallow}/20250614_102301.jpg": "",
        f"{assets2_bluer_swallow}/20250614_114954.jpg": "",
        f"{assets2_bluer_swallow}/20250615_192339.jpg": "",
        f"{assets2_bluer_swallow}/20250703_153834.jpg": "",
        f"{assets2_bluer_swallow}/design/v2/01.jpg": "",
    }
)


docs = [
    {
        "path": "../docs/bluer_swallow/digital/design/shield.md",
        "items": items,
    },
]
